/// Circle

struct Circle {
    Point<> c;
    double r;

    Circle(Point<> c, double r) : c(c), r(r) {}

    // Circumcircle
    Circle(Point<> a, Point<> b, Point<> c) {
        Point<> u((b - a).y, -(b - a).x);
        Point<> v((c - a).y, -(c - a).x);
        Point<> n = (c - b)*0.5;

        double t = u.cross(n) / v.cross(u);

        this->c = (a + c)*0.5 + v*t;
        this->r = dist(this->c, a);
    }

    // Minimum enclosing circle: O(n)
    Circle(vector<Point<>> p) {
        random_shuffle(all(p));
        Circle C(p[0], 0.0);

        for (int i = 0; i < p.size(); ++i) {
            if (C.contains(p[i])) continue;
            C = Circle(p[i], 0.0);

            for (int j = 0; j < i; ++j) {
                if (C.contains(p[j])) continue;
                C = Circle((p[j] + p[i])*0.5, 0.5*dist(p[j], p[i]));

                for (int k = 0; k < j; ++k) {
                    if (C.contains(p[k])) continue;
                    C = Circle(p[j], p[i], p[k]);
                }
            }
        }

        this->c = C.c;
        this->r = C.r;
    }

    bool contains(Point<> p) {
        return (dist(c, p) <= r + EPS);
    }
};
