/// Convex Hull
///
/// Time: O(n log n)
/// Space: O(n)

template <typename T>
bool cw(Point<T> a, Point<T> b, Point<T> c) {
    return (b - a).cross(c - a) <= 0;
}

template <typename T>
vector<Point<T>> convex_hull(vector<Point<T>> &v) {
    int k = 0;
    vector<Point<T>> ans(v.size() * 2);

    sort(all(v), [](const Point<T> &a, const Point<T> &b) {
        return (a.x == b.x) ? (a.y < b.y) : (a.x < b.x);
    });

    for (int i = 0; i < v.size(); ++i) {
        for (; k >= 2 && cw(ans[k-2], ans[k-1], v[i]); --k);
        ans[k++] = v[i];
    }

    for (int i = v.size() - 2, t = k + 1; i >= 0; --i) {
        for (; k >= t && cw(ans[k-2], ans[k-1], v[i]); --k);
        ans[k++] = v[i];
    }

    ans.resize(k);
    return ans;
}
