/// Point

template <typename T = double>
struct Point {
    T x, y; 

    Point() {}
    Point(T x, T y) : x(x), y(y) {}

    Point operator+(Point p) { return Point(x+p.x, y+p.y); }
    Point operator-(Point p) { return Point(x-p.x, y-p.y); }
    Point operator*(T s) { return Point(x*s, y*s); }

    T dot(Point p)   { return (x*p.x) + (y*p.y); }
    T cross(Point p) { return (x*p.y) - (y*p.x); }

    // Returns angle between this and p:
    // atan2(y, x) is in the range [-180,180]. To 
    // get [0, 360], atan2(-y, -x) + 180 is used
    T angle(Point p) {
        return to_deg(atan2(-cross(p), -dot(p))) + 180.0;
    }

    // Returns cosine value between this and p.
    T cosine(Point p) {
        return (dot(p) / (sqrt(dot(*this))*sqrt(p.dot(p))));
    }

    // Returns sine value between this and p.
    T sine(Point p) {
        return (cross(p) / (sqrt(dot(*this))*sqrt(p.dot(p))));
    }

    // Returns whether point is inside the triable
    // abc or not.
    bool inside_triagle(Point a, Point b, Point c) {
        bool c1 = (*this - b).cross(a - b) < 0;
        bool c2 = (*this - c).cross(b - c) < 0;
        bool c3 = (*this - a).cross(c - a) < 0;
        return c1 == c2 && c1 == c3;
    }

    // Finds orientation of ordered triplet (a,b,c).
    // Colinear (0), Clockwise (1), Counterclockwise (2)
    static int orientation(Point a, Point b, Point c) {
        T val = (b - a).cross(c - b);
        if (val == 0) return 0;
        return (val > 0) ? 1 : 2;
    }
};
