/// Polygon

template <typename T = double>
struct Polygon {
    vector<Point<T>> v;

    Polygon() {}
    Polygon(vector<Point<T>> v) : v(v) {} 

    // Adds a vertex to the polygon.
    void add_point(Point<T> p) { v.pb(p); }

    // Returns area of polygon (only works when vertices
    // are sorted in clockwise or counterclockwise order).
    double area() {
        double ans = 0;
        for (int i = 0; i < v.size(); ++i)
            ans += v[i].cross(v[(i + 1) % v.size()]);

        return fabs(ans) / 2.0;
    }

    // Rotating Calipers
    double width() {
        vector<Point<T>> h = convex_hull(v);

        int n = h.size() - 1;
        double ans = 1e14;

        h[0] = h[n];
        for (int i = 1, j = 1; i <= n; ++i) {
            while ((h[i] - h[i-1]).cross(h[j%n+1] - h[i-1]) > 
                    (h[i] - h[i-1]).cross(h[j] - h[i-1]))
                j = j % n + 1;

            Segment<T> seg(h[i], h[i-1]);
            ans = min(ans, seg.dist(h[j]));
        }

        return ans;
    }

    // Rotating Calipers
    double diameter() {
        vector<Point<T>> h = convex_hull(v);

        if (h.size() == 1) return 0;
        if (h.size() == 2) return dist(h[0], h[1]);

        int n = h.size() - 1;
        double ans = -1e14;

        h[0] = h[n];
        for (int i = 1, j = 1; i <= n; ++i) {
            while ((h[i] - h[i-1]).cross(h[j%n+1] - h[i-1]) > 
                    (h[i] - h[i-1]).cross(h[j] - h[i-1]))
                j = j % n + 1;

            ans = max(ans, dist(h[j], h[i]));
            ans = max(ans, dist(h[j], h[i-1]));
        }

        return ans;
    }
};
