/// Segment

template <typename T = double>
struct Segment {
    Point<T> a, b;

    Segment(Point<T> a, Point<T> b) : a(a), b(b) {}

    // Checks if points p and q are on the same side 
    // of the segment.
    bool same_side(Point<T> p, Point<T> q) { 
        T cpp = (p - a).cross(b - a);
        T cpq = (q - a).cross(b - a);
        return ((cpp > 0 && cpq > 0) || 
                (cpp < 0 && cpq < 0));
    }

    // Checks if point p is on the segment.
    bool on_segment(Point<T> p) {
        return (p.x <= max(a.x, b.x) && 
                p.x >= min(a.x, b.x) &&
                p.y <= max(a.y, b.y) && 
                p.y >= min(a.y, b.y));
    }

    // Distance between segment and point
    double dist(Point<T> p) {
        return (a - b).cross(p - b)/sqrt((b - a).dot(b - a));
    }

    // Checks if segment intersects with s.
    bool intersect(Segment<T> s) {
        int o1 = Point<T>::orientation(  a,   b, s.a);
        int o2 = Point<T>::orientation(  a,   b, s.b);
        int o3 = Point<T>::orientation(s.a, s.b,   a);
        int o4 = Point<T>::orientation(s.a, s.b,   b);

        if (o1 != o2 && o3 != o4)
            return true;

        if (o1 == 0 && on_segment(s.a)) return true;
        if (o2 == 0 && on_segment(s.b)) return true;
        if (o3 == 0 && s.on_segment(a)) return true;
        if (o4 == 0 && s.on_segment(b)) return true;

        return false;
    }
};
