/// Bellman-Ford
///
/// Time: O(V * E)
/// Space: O(V + E)
///
/// Status: Tested (LA4509)

struct BellmanFord {
    struct Edge { int u, v, w; };

    int N;
    vector<int> dist;
    vector<Edge> graph;

    BellmanFord(int N) : 
        N(N), dist(N)
    { init(); }

    void init() {
        fill(all(dist), inf);
    }

    void add_edge(int u, int v, int w) {
        graph.pb({ u, v, w });
    }

    int run(int s, int d) {
        dist[s] = 0;
        for (int i = 0; i < N; ++i)
            for (auto e : graph)
                if (dist[e.u] != inf && 
                        dist[e.u] + e.w < dist[e.v])
                    dist[e.v] = dist[e.u] + e.w;

        // Check for negative cycles, return -inf if 
        // there is one
        for (auto e : graph)
            if (dist[e.u] != inf && 
                    dist[e.u] + e.w < dist[e.v])
                return -inf;

        return dist[d];
    }
};
