/// Dijkstra
///
/// Description:
///   Dijkstra's algorithm for finding the shortest paths between nodes in a 
/// graph. It works by greedily extending the shortest path at each step. \par
///   Doesn't work with negative-weighted edges, for that, Bellman-Ford 
/// algorithm must be used.
///
/// Time: O(E + V log V)
/// Space: O(V + E)

vector<int> graph[MAX];

struct Dijkstra {
    int N;
    vector<int> dist, vis;

    Dijkstra(int N) :
        N(N), dist(N), vis(N) 
    { init(); }

    void init() {
        fill(all(vis), 0);
        fill(all(dist), inf);
    }

    int run(int s, int d) {
        set<p<int>> pq;
        dist[s] = 0;
        pq.insert({0, s});

        while (pq.size() != 0) {
            int u = pq.begin()->se;
            pq.erase(pq.begin());

            if (vis[u]) continue;
            vis[u] = 1;

            for (auto i : graph[u]) {
                if (!vis[i.fi] && dist[i.fi] > dist[u] + i.se) {
                    dist[i.fi] = dist[u] + i.se;
                    pq.insert({dist[i.fi], i.fi});
                }
            }
        }

        return dist[d];
    }
};
