/// Dinic's
///
/// Time: O(E * V^2)
/// Space: O(V + E)
///
/// Status: Tested (URI2882,URI2354)

struct Dinic {
    struct Edge { int u, f, c, r; };

    int N;
    vector<int> depth, start;
    vector<vector<Edge>> graph;

    Dinic(int N) : 
        N(N), depth(N), 
        start(N), graph(N) 
    {}

    void add_edge(int u, int v, int c) {
        Edge forw = { v, 0, c, (int) graph[v].size() };
        Edge back = { u, 0, 0, (int) graph[u].size() };
        graph[u].pb(forw);
        graph[v].pb(back);
    }

    bool bfs(int s, int t) {
        queue<int> Q;
        Q.push(s);

        fill(all(depth), -1);
        depth[s] = 0;

        while (!Q.empty()) {
            int v = Q.front(); Q.pop();
            for (auto i : graph[v])
                if (depth[i.u] == -1 && i.f < i.c) {
                    depth[i.u] = depth[v] + 1;
                    Q.push(i.u);
                }
        }
        return depth[t] != -1;
    }

    int dfs(int s, int t, int f) {
        if (s == t) return f;

        for ( ; start[s] < graph[s].size(); ++start[s]) {
            Edge &e = graph[s][start[s]];
            if (depth[e.u] == depth[s] + 1 && e.f < e.c) {
                int min_f = dfs(e.u, t, min(f, e.c - e.f));
                if (min_f > 0) {
                    e.f += min_f;
                    graph[e.u][e.r].f -= min_f;
                    return min_f;
                }
            }
        }
        return 0;
    }

    int run(int s, int t) {
        int ans = 0;
        while (bfs(s, t)) {
            fill(all(start), 0);
            while (int flow = dfs(s, t, inf))
                ans += flow;
        }
        return ans;
    }
};
