/// Edmonds-Karp
///
/// Time: O(V * E^2)
/// Space: O(V^2)
///
/// Status: Tested (URI2354)

int rg[MAX][MAX];
int graph[MAX][MAX];

struct EdmondsKarp {
    int N;
    vector<int> par, vis;

    EdmondsKarp(int N) :
        N(N), par(N), vis(N) 
    { init(); }

    void init() {
        fill(all(vis), 0);
    }

    bool bfs(int s, int t) {
        queue<int> Q;
        Q.push(s);
        vis[s] = true;

        while (!Q.empty()) {
            int u = Q.front(); Q.pop();
            if (u == t) return true;
            for (int i = 0; i < N; ++i)
                if (!vis[i] && rg[u][i]) {
                    vis[i] = true;
                    par[i] = u;
                    Q.push(i);
                }
        }
        return false;
    }

    int run(int s, int t) {
        int ans = 0;
        par[s] = -1;
        memcpy(rg, graph, sizeof(graph));

        while (bfs(s, t)) {
            int flow = inf;
            for (int i = t; par[i] != -1; i = par[i])
                flow = min(flow, rg[par[i]][i]);

            for (int i = t; par[i] != -1; i = par[i]) {
                rg[par[i]][i] -= flow;
                rg[i][par[i]] += flow;
            }
            ans += flow;
            init();
        }
        return ans;
    }
};
