/// Ford-Fulkerson
/// 
/// Time: O(E * f)
/// Space: O(V^2)
///
/// Status: Tested (URI2354)

int rg[MAX][MAX];
int graph[MAX][MAX];

struct FordFulkerson {
    int N;
    vector<int> par, vis;

    FordFulkerson(int N) :
        N(N), par(N), vis(N)
    { init(); }

    void init() {
        fill(all(vis), 0);
    }

    bool dfs(int s, int t) {
        vis[s] = true;
        if (s == t) return true;

        for (int i = 0; i < N; ++i)
            if (!vis[i] && rg[s][i]) {
                par[i] = s;
                if (dfs(i, t)) return true;
            }
        return false;
    }

    int run(int s, int t) {
        int ans = 0;
        par[s] = -1;
        memcpy(rg, graph, sizeof(graph));

        while (dfs(s, t)) {
            int flow = inf;
            for (int i = t; par[i] != -1; i = par[i])
                flow = min(flow, rg[par[i]][i]);

            for (int i = t; par[i] != -1; i = par[i]) {
                rg[par[i]][i] -= flow;
                rg[i][par[i]] += flow;
            }
            ans += flow;
            init();
        }
        return ans;
    }
};
