/// Hopcroft-Karp
///
/// Time: O(E * sqrt{V})
/// Space: O(V + E)
///
/// Caution:
///   - Assumes 0-indexed vertices in graph;
///   - The vertex enumeration on the left CAN be the same as the right.
///
/// Status: Tested (GCJ18-R2C)

vector<int> graph[MAX];

struct HopcroftKarp {
    int L, R;
    vector<int> dist;
    vector<int> matchL, matchR;

    HopcroftKarp(int L, int R) :
        L(L), R(R), dist(L),
        matchL(L), matchR(R)
    { init(); }

    void init() {
        fill(all(matchL), -1);
        fill(all(matchR), -1);
    }

    bool bfs() {
        queue<int> Q;
        for (int l = 0; l < L; ++l)
            if (matchL[l] == -1) {
                dist[l] = 0;
                Q.push(l);
            } else
                dist[l] = -1;

        bool ans = false;
        while (!Q.empty()) {
            int l = Q.front(); Q.pop();
            for (auto r : graph[l])
                if (matchR[r] == -1)
                    ans = true;
                else if (dist[matchR[r]] == -1) {
                    dist[matchR[r]] = dist[l] + 1;
                    Q.push(matchR[r]);
                }
        }

        return ans;
    }

    bool dfs(int l) {
        if (l == -1)
            return true;

        for (auto r : graph[l])
            if (matchR[r] == -1 ||
                    dist[matchR[r]] == dist[l] + 1)
                if (dfs(matchR[r])) {
                    matchR[r] = l;
                    matchL[l] = r;
                    return true;
                }

        return false;
    }

    int run() {
        int ans = 0;
        while (bfs())
            for (int l = 0; l < L; ++l)
                if (matchL[l] == -1 && dfs(l))
                    ans++;

        return ans;
    }
};
