/// Kosaraju
///
/// Time: O(V + E)
/// Space: O(V + E)

vector<int> graph[MAX];
vector<int> transp[MAX];

struct Kosaraju {
    int N;
    stack<int> S;
    vector<int> vis;

    Kosaraju(int N) : 
        N(N), vis(N) 
    { init(); }

    void init() { 
        fill(all(vis), 0); 
    }

    void dfs(int x) {
        vis[x] = true;
        for (auto i : transp[x])
            if (!vis[i])
                dfs(i);
    }

    // Fills stack with DFS starting points to find SCC.
    void fill_stack(int x) {
        vis[x] = true;
        for (auto i : graph[x])
            if (!vis[i])
                fill_stack(i);
        S.push(x);
    }

    int run() {
        int scc = 0;
        for (int i = 0; i < N; ++i)
            if (!vis[i])
                fill_stack(i);

        // Transpose graph
        for (int i = 0; i < N; ++i)
            for (auto j : graph[i])
                transp[j].push_back(i);
        init();

        // Count SCC
        while (!S.empty()) {
            int v = S.top();
            S.pop();
            if (!vis[v]) {
                dfs(v);
                scc++;
            }
        }

        return scc;
    }
};
