/// Kruskal
///
/// Time: O(E log V) 
/// Space: O(E)
/// 
/// Include:
///  - structure/disjoint_set

using edge = pair<p<int>,int>;
vector<edge> edges;

struct Kruskal {
    int N;
    DisjointSet ds;

    Kruskal(int N) : N(N), ds(N) {}

    // Minimum Spanning Tree: comp = less<int>()
    // Maximum Spanning Tree: comp = greater<int>()
    int run(vector<edge> &mst, function<bool(int,int)> comp) {
        sort(all(edges), [&](const edge &a, const edge &b) {
                return comp(a.se, b.se);
                });

        int size = 0;
        for (int i = 0; i < edges.size(); i++) {
            int pu = ds.find_set(edges[i].fi.fi);
            int pv = ds.find_set(edges[i].fi.se);

            if (pu != pv) {
                mst.pb(edges[i]);
                size += edges[i].se;
                ds.union_set(pu, pv);
            }
        }

        return size;
    }
};
