/// Minimum Cost Maximum Flow
///
/// Time: O(V^2 * E)
/// Space: O(V + E)
///
/// Status: Tested (UVA10594,kattis/mincostmaxflow)

struct MinCostMaxFlow {
    struct Edge { int u, v, cap, cost; };

    vector<Edge> edges;
    vector<vector<int>> adj;
    vector<int> vis, dist, par, ind;

    MinCostMaxFlow(int N) :
        vis(N), dist(N), par(N), ind(N), adj(N) {}

    void add_edge(int u, int v, int cap, int cost) {
        adj[u].pb(edges.size());
        edges.pb({ u, v, cap, cost });
        adj[v].pb(edges.size());
        edges.pb({ v, u, 0, -cost });
    }

    // Shortest Path Faster Algorithm (slower than 
    // Dijkstra but works with negative edges).
    bool spfa(int s, int t) {
        fill(all(dist), inf);
        dist[s] = 0;
        queue<int> Q;
        Q.push(s);

        while (!Q.empty()) {
            int u = Q.front(); Q.pop();
            vis[u] = 0;

            for (auto i : adj[u]) {
                Edge &e = edges[i];
                int v = e.v;
                if (e.cap > 0 && dist[v] > dist[u] + e.cost) {
                    dist[v] = dist[u] + e.cost;
                    par[v] = u;
                    ind[v] = i;

                    if (!vis[v]) {
                        Q.push(v);
                        vis[v] = 1;
                    }
                }
            }
        }
        return dist[t] < inf;
    }

    // Returns pair (min_cost, max_flow).
    p<int> run(int s, int t) {
        int min_cost = 0;
        int max_flow = 0;

        while (spfa(s, t)) {
            int flow = inf;
            for (int i = t; i != s; i = par[i])
                flow = min(flow, edges[ind[i]].cap);

            for (int i = t; i != s; i = par[i]) {
                edges[ind[i]  ].cap -= flow;
                edges[ind[i]^1].cap += flow;
            }

            min_cost += flow * dist[t];
            max_flow += flow;
        }

        return {min_cost, max_flow};
    }
};
