/// Tarjan
///
/// Time: O(V + E)
/// Space: O(V + E)

vector<int> scc[MAX];
vector<int> graph[MAX];

struct Tarjan {
    int N, ncomp, ind;

    stack<int> S;
    vector<int> vis, id, low;

    Tarjan(int N) :
        N(N), vis(N), id(N), low(N)
    { init(); }

    void init() {
        fill(all(id), -1);
        fill(all(vis), 0);
    }

    void dfs(int x) {
        id[x] = low[x] = ind++;
        vis[x] = 1;
        S.push(x);

        for (auto i : graph[x])
            if (id[i] == -1) {
                dfs(i);
                low[x] = min(low[x], low[i]);
            } else if (vis[i])
                low[x] = min(low[x], id[i]);

        // A SCC was found
        if (low[x] == id[x]) {
            int w;
            do {
                w = S.top(); S.pop();
                vis[w] = 0;
                scc[ncomp].pb(w);
            } while (w != x);
            ncomp++;
        }
    }

    int run() {
        init();
        ncomp = ind = 0;
        for (int i = 0; i < N; ++i)
            scc[i].clear();

        // Apply tarjan in every component
        for (int i = 0; i < N; ++i)
            if (id[i] == -1)
                dfs(i);
        return ncomp;
    }
};
