/// Topological Sort
///
/// Time: O(V + E)
/// Space: O(V + E)

vector<int> graph[MAX];

struct TopologicalSort {
    int N;
    stack<int> S;
    vector<int> vis;

    TopologicalSort(int N) : 
        N(N), vis(N) 
    { init(); }

    void init() {
        fill(all(vis), 0);
    }

    bool dfs(int x) {
        vis[x] = 1;
        for (auto i : graph[x]) {
            if (vis[i] == 1) return true;
            if (!vis[i] && dfs(i)) return true;
        }

        vis[x] = 2;
        S.push(x);
        return false;
    }

    // Returns whether graph contains cycle 
    // or not.
    bool run(vector<int> &tsort) {
        init();
        bool cycle = false;
        for (int i = 0; i < N; ++i)
            if (!vis[i])
                cycle |= dfs(i);
        if (cycle) return true;

        while (!S.empty()) {
            tsort.pb(S.top());
            S.pop();
        }
        return false;
    }
};
