/// Travelling Salesman
///
/// Description:
///   Given a graph and an origin vertex, this algorithm returns the shortest
/// possible route that visits each vertex and returns to the origin. \par
///   The algorithm works by using dynamic programming, where dp[i][mask]
/// stores the last visited vertex $i$ and a set of visited vertices 
/// represented by a bitmask mask. Given a state, the next vertex in the path
/// is chosen by a recursive call, until the bitmask is full, in which case the
/// weight of the edge between the last vertex and the origin in returned.
///
/// Time: O(2^n * n^2)
/// Space: O(2^n * n)
/// 
/// Status: Tested (Uva10496)

int dp[MAX][1 << MAX];
int graph[MAX][MAX];

struct TSP {
    int N;

    TSP(int N) : N(N)
    { init(); }

    void init() { mset(dp, -1); }

    int solve(int i, int mask) {
        if (mask == (1 << N) - 1)
            return graph[i][0];
        if (dp[i][mask] != -1)
            return dp[i][mask];

        int ans = inf;
        for (int j = 0; j < N; ++j)
            if (!(mask & (1 << j)) && (i != j))
                ans = min(ans, graph[i][j] + 
                        solve(j, mask | (1 << j)));
        return dp[i][mask] = ans;
    }

    int run(int start) {
        return solve(start, 1 << start);
    }
};
