/// Binary Exponentiation
/// 
/// Description:
///   Computes fast exponentiation by looking a the binary representation of
/// the exponent. The usage of ``bin\_mul'' is not necessary, but it ensures 
/// that no overflow will occur when multiplying two large integers. It is
/// definitely required in order to work with Miller-Rabin and Pollard's
/// Rho implementations (when dealing with numbers that might go up to 
/// $10^{18}$. USE BIN\_MUL ONLY IF NECESSARY (VERY SLOW).
///
/// Time: O(log n)
/// Space: O(1)

i64 bin_mul(i64 a, i64 b, i64 M = MOD) {
    i64 x = 0;
    a %= M;
    while (b) {
        if (b & 1) x = (x + a) % M;
        b >>= 1;
        a = (a * 2) % M;
    }
    return x % M;
}

i64 bin_exp(i64 a, i64 e, i64 M = MOD) {
    i64 x = 1;
    while (e) {
        if (e & 1) 
            x = (x * a) % M;
        e >>= 1;
        a = (a * a) % M;
    }
    return x % M;
}
