/// Karatsuba
///
/// Time: O(n^{log(3)})
/// Space: O(n)

vector<i64> karatsuba(const vector<i64> &a, 
        const vector<i64> &b) 
{
    int n = a.size();
    vector<i64> res(n + n);

    if (n <= 32) {
        for (int i = 0; i < n; i++)
            for (int j = 0; j < n; j++)
                res[i + j] += a[i] * b[j];

        return res;
    }

    int k = n >> 1;
    vector<i64> a1(a.begin(), a.begin() + k);
    vector<i64> a2(a.begin() + k, a.end());
    vector<i64> b1(b.begin(), b.begin() + k);
    vector<i64> b2(b.begin() + k, b.end());

    vector<i64> a1b1 = karatsuba(a1, b1);
    vector<i64> a2b2 = karatsuba(a2, b2);

    for (int i = 0; i < k; i++)
        a2[i] += a1[i];
    for (int i = 0; i < k; i++)
        b2[i] += b1[i];

    vector<i64> r = karatsuba(a2, b2);
    for (int i = 0; i < a1b1.size(); i++) 
        r[i] -= a1b1[i];
    for (int i = 0; i < a2b2.size(); i++) 
        r[i] -= a2b2[i];

    for (int i = 0; i < r.size(); i++) 
        res[i + k] += r[i];
    for (int i = 0; i < a1b1.size(); i++) 
        res[i] += a1b1[i];
    for (int i = 0; i < a2b2.size(); i++) 
        res[i + n] += a2b2[i];

    return res;
}
