/// Legendre's Formula
///
/// Description:
///   Given an interger $n$ and a prime number $p$, find the largest power of
/// $p$ ($x$) such that $n!$ is divisible by $p^x$. \par
///   Writing $n!$ as $1 \cdot 2 \cdot 3 ... (n-1) \cdot n$ shows that every
/// $k$-th element of the product is divisible by $k$ (i.e. adds $1$ to the
/// answer), the number of such elements is $\lfloor\frac{n}{k}\rfloor$. The 
/// same is true for every $k^i$, thus, the answer is given by: 
/// \[ \lfloor\frac{n}{k}\rfloor + \lfloor\frac{n}{k^2}\rfloor + ... + 
/// \lfloor\frac{n}{k^i}\rfloor \] only approximately the first $log_p n$
/// elements are greater than zero, therefore the sum is finite.
///
/// Time: O(log_p n)
/// Space: O(1)

int legendre(int n, int p) {
    int x = 0;

    while (n) {
        n /= p;
        x += n;
    }

    return x;
}
