/// Miller-Rabin primality test
///
/// Time: O(k * log^3 n)
/// Space: O(1)

const vector<i64> A = {
    2, 325, 9375, 28178, 450775, 9780504, 1795265022
};

bool is_prime(i64 n) {
    if (n < 2 || n % 6 % 4 != 1)
        return n - 2 < 2;

    i64 s = __builtin_ctzll(n - 1);
    i64 d = n >> s;

    for (auto a : A) {
        i64 p = bin_exp(a, d, n), i = s;
        while (p != 1 && p != n - 1 && a % n && i--)
            p = bin_mul(p, p, n);
        if (p != n - 1 && i != s) 
            return 0;
    }

    return 1;
}
