/// Modular Multiplicative Inverse
///
/// Description:
///   Given integers $a$ and $m$, the modular multiplicative inverse of $a$ is
/// an integer $x$ such that \[ a \cdot x \equiv 1 \pmod{m} \] also denoted as
/// $a^{-1}$.
///
/// Time: O(log m)
/// Space: O(1)

// Fermat's Little Theorem: Used when m is prime
int fermat(int a, int m) {
    return bin_exp(a, m - 2);
}

// Extended Euclidean Algorithm: Used when m 
// and a are coprime
int extended_euclidean(int a, int m) {
    ans_t g = ext_gcd(a, m);
    return (g.x % m + m) % m;
}
