/// Pollard's Rho
///
/// Description: 
///   Returns prime factorization of $n$.
///
/// Time: O(n^{1/4})
/// Space: O(1)

i64 pollard(i64 n) {
    auto f = [n](i64 x) {
        return (bin_mul(x, x, n) + 1) % n;
    };

    if (n % 2 == 0) return 2;

    for (i64 i = 2; ; ++i) {
        i64 x = i, y = f(x), p;
        while ((p = __gcd(n + y - x, n)) == 1)
            x = f(x), y = f(f(y));
        if (p != n) return p;
    }
}

vector<i64> factor(i64 n) {
    if (n == 1) return {};
    if (is_prime(n)) // Use Miller-Rabin
        return {n};

    i64 x = pollard(n);
    auto l = factor(x);
    auto r = factor(n/x);

    l.insert(l.end(), all(r));
    return l;
}
