/// Ternary Search
///
/// Description:
///   Searches for the minimum (or maximum) value of a unimodal function
/// $f(x)$ for $x$ between $l$ and $r$.
///
/// Time: O(log n)
/// Space: O(1)
///
/// Caution:
///   - f must be a unimodal function

// Minimum of f: comp = less<T>()
// Maximum of f: comp = greater<T>()
template <typename T, class F = function<T(T)>>
T ternary_search(T l, T r, F f, function<bool(T,T)> comp) {
    T rt, lt;

    for (int i = 0; i < 500; ++i) {
        if (fabs(r - l) < EPS)
            return (l + r) / 2;

        lt = (r - l) / 3 + l;
        rt = ((r - l) * 2) / 3 + l;

        if (comp(f(lt), f(rt)))
            l = lt;
        else
            r = rt;
    }

    return (l + r) / 2;
}
