/// Booth's Algorithm
///
/// Description:
///   This algorithm finds the lexicographically minimal (or maximal) string
/// rotation, that is, given a string $s$, the goal is to find a rotation of 
/// $s$ possessing the lowest (or highest) lexicographical order among all 
/// possible rotations.
///
/// Time: O(n)
/// Space: O(n)

// Maximal: func = greater<char>()
// Minimal: func = less<char>()
string booth(string s, function<bool(char,char)> func) {
    string S = s + s;
    vector<int> f(S.size(), -1);

    int k = 0;
    for (int j = 1; j < S.size(); ++j) {
        char sj = S[j];
        int i = f[j - k - 1];

        while (i != -1 && sj != S[k+i+1]) {
            if (func(sj, S[k + i + 1]))
                k = j - i - 1;
            i = f[i];
        }

        if (sj != S[k+i+1]) {
            if (func(sj, S[k]))
                k = j;

            f[j-k] = -1;
        } else
            f[j-k] = i + 1;
    }

    string ans(s.size(), 0);
    for (int i = 0; i < s.size(); ++i)
        ans[i] = S[k+i];
    return ans;
}
