/// Knuth-Morris-Pratt (KMP)
///
/// Time: 
///   - preprocess: O(m)
///   - search:     O(n)
/// Space: O(n + m)
///
/// Status: Tested (URI2350,UVA10679)

struct KMP {
    string patt;
    vector<int> pi;

    KMP(string patt) :
        patt(patt), pi(patt.size())
    { preprocess(); }

    void preprocess() {
        pi[0] = 0;
        for (int i = 1, j = 0; i < patt.size(); ++i) {
            while (j > 0 && patt[i] != patt[j]) 
                j = pi[j - 1];

            if (patt[i] == patt[j]) j++;
            pi[i] = j;
        }
    }

    void search(const string &txt) {
        for (int i = 0, j = 0; i < txt.size(); ++i) {
            while (j > 0 && txt[i] != patt[j]) 
                j = pi[j - 1];

            if (txt[i] == patt[j]) j++;
            if (j == patt.size()) {
                cout << "Pattern found at " << (i - j) << ende;
                j = pi[j - 1];
            }
        }
    }
};
