/// AVL tree
///
/// Time: O(log n)
/// Space: O(n)

struct AVL {
    struct Node {
        int key, height;
        Node *left, *right;

        Node(int k) :
            key(k), height(1),
            left(nullptr), right(nullptr)
        {}

        static int get_height(Node *n) {
            return (n == nullptr) ? 0 : n->height;
        }

        void fix_state() {
            height = max(Node::get_height(left), 
                    Node::get_height(left)) + 1;
        }

        int get_balance() {
            return Node::get_height(left) - 
                Node::get_height(right);
        }
    };

    Node *root;

    AVL() : root(nullptr) {}

    void insert(int key) {
        root = insert(root, key);
    }

    private:

    Node *rotate_right(Node *n) {
        Node *aux1 = n->left;
        Node *aux2 = aux1->right;
        aux1->right = n; aux1->fix_state();
        n->left = aux2; n->fix_state();
        return aux1;
    }

    Node *rotate_left(Node *n) {
        Node *aux1 = n->right;
        Node *aux2 = aux1->left;
        aux1->left = n; aux1->fix_state();
        n->right = aux2; n->fix_state();
        return aux1;
    }

    Node *insert(Node *n, int key) {
        if (n == nullptr) {
            Node *new_node = new Node(key);
            if (root == nullptr) root = new_node;
            return new_node;
        }

        if (key < n->key)
            n->left  = insert(n->left, key);
        else
            n->right = insert(n->right, key);

        int balance = n->get_balance();
        n->fix_state();

        if (balance > 1 && key < n->left->key) {
            return rotate_right(n);
        } else if (balance < -1 && key > n->right->key) {
            return rotate_left(n);
        } else if (balance > 1 && key > n->left->key) {
            n->left = rotate_left(n->left);
            return rotate_right(n);
        } else if (balance < -1 && key < n->right->key) {
            n->right = rotate_right(n->right);
            return rotate_left(n);
        }

        return n;
    }
};
