/// Binary Indexed Tree (BIT)
///
/// Time:
///   - update: O(log n)
///   - query:  O(log n)
/// Space: O(n)

struct BIT {
    int N;
    vector<int> tree;

    BIT(int N) : N(N), tree(N, 0) {}

    int query(int idx) {
        int sum = 0;
        for (; idx > 0; idx -= (idx & -idx))
            sum += tree[idx];
        return sum;
    }

    void update(int idx, int val) {
        for (; idx < N; idx += (idx & -idx))
            tree[idx] += val;
    }
};
