/// Binary Indexed Tree 2D (BIT2D)
///
/// Time:
///   - update: O(log^2 n)
///   - query:  O(log^2 n)
/// Space: O(n^2)
///
/// Status: Tested (MCAIRO)

struct BIT2D {
    int N, M;
    vector<vector<int>> tree;

    BIT2D(int N, int M) : N(N), M(M), 
    tree(N, vector<int>(M, 0)) 
    {}

    int query(int idx, int idy) {
        int sum = 0;
        for (; idx > 0; idx -= (idx & -idx))
            for (int m = idy; m > 0; m -= (m & -m))
                sum += tree[idx][m];
        return sum;
    }

    void update(int idx, int idy, int val) {
        for (; idx < N; idx += (idx & -idx))
            for (int m = idy; m < M; m += (m & -m))
                tree[idx][m] += val;
    }
};
