/// Disjoint-set
///
/// Time:
///   - make_set:  O(1)
///   - find_set:  O(a(n))
///   - union_set: O(a(n))
/// Space: O(n)

struct DisjointSet {
    int N;
    vector<int> rnk, par, siz;

    DisjointSet(int N) :
        N(N), rnk(N), par(N), siz(N)
    { init(); }

    void init() {
        iota(all(par), 0);
        fill(all(rnk), 0);
        fill(all(siz), 1);
    }

    int find_set(int x) {
        if (par[x] != x)
            par[x] = find_set(par[x]);
        return par[x];
    }

    void union_set(int x, int y) {
        x = find_set(x);
        y = find_set(y);

        if (x == y) return;
        if (rnk[x] < rnk[y]) swap(x, y);
        if (rnk[x] == rnk[y]) rnk[x]++;
        par[y] = x;
        siz[x] += siz[y];
    }
};
