/// Hash Function (splitmix64)
///
/// Description:
///   The default hash function for unordered\_map in C++ is not reliable when
/// it comes to non-random input (i.e. input designed to cause collisions, 
/// such as multiples of a specific prime). This custom hash function solves 
/// this problem (https://codeforces.com/blog/entry/62393).
///
/// Time:  O(1)
/// Space: O(n)

struct CustomHash {
    static uint64_t splitmix64(uint64_t x) {
        x += 0x9e3779b97f4a7c15;
        x = (x ^ (x >> 30)) * 0xbf58476d1ce4e5b9;
        x = (x ^ (x >> 27)) * 0x94d049bb133111eb;
        return x ^ (x >> 31);
    }

    size_t operator()(uint64_t x) const {
        static const uint64_t FIXED_RANDOM = chrono::steady_clock::now()
            .time_since_epoch()
            .count();
        return splitmix64(x + FIXED_RANDOM);
    }
};

unordered_map<i64,int,CustomHash> M;
