/// Segment Tree
///
/// Time:
///   - update: O(log n)
///   - query:  O(log n)
/// Space: O(n)
///
/// Caution:
///   - Provide identity value if necessary (default is T())

template<typename T, class F = function<T(T,T)>>
struct SegmentTree {
    int N;
    F func;
    T id = T();
    vector<T> tree;

    SegmentTree(int N, F func) : 
        N(N), func(func), tree(N*2, id) 
    {}

    void build(const vector<T> &v) {
        for (int i = 0; i < N; ++i)
            tree[N + i] = v[i];
        for (int i = N - 1; i > 0; --i) 
            tree[i] = func(tree[i*2], tree[i*2+1]);
    }

    void update(int i, T x) {
        tree[i += N] = x;
        for (i /= 2; i > 0; i /= 2)
            tree[i] = func(tree[i*2], tree[i*2+1]);
    }

    T query(int i, int j) {
        T left = id, right = id;
        for (i += N, j += N; i < j; i /= 2, j /= 2) {
            if (i & 1) left = func(left, tree[i++]);
            if (j & 1) right = func(right, tree[--j]);
        }
        return func(left, right);
    }
};
