/// Sparse Table
///
/// Time:
///   - preprocess:  O(n log n)
///   - query:  O(1)
/// Space: O(n log n)
///
/// Caution:
///   - func must be min, max, gcd, or lcm
///   - Query computes for [l,r]

#define LOG 20

template <typename T, class F = function<T(T,T)>>
struct SparseTable {
    F func;
    int N;
    vector<T> log;
    vector<vector<T>> table;

    SparseTable(const vector<T> &v, F func) :
        N(v.size()),
        log(N + 1), 
        table(N, vector<T>(LOG+1)), 
        func(func)
    { preprocess(v); }

    void preprocess(const vector<T> &v) {
        log[1] = 0;
        for (int i = 2; i <= N; ++i)
            log[i] = log[i >> 1] + 1;

        for (int i = 0; i < N; ++i)
            table[i][0] = v[i];

        for (int j = 1; j <= LOG; ++j)
            for (int i = 0; i + (1 << j) <= N; ++i)
                table[i][j] = func(table[i][j-1], table[i+(1<<(j-1))][j-1]);
    }

    T query(int l, int r) {
        int j = log[r - l + 1];
        return func(table[l][j], table[r - (1<<j) + 1][j]);
    }
};
