/// Sqrt Decomposition
///
/// Time:
///   - preprocess: O(n)
///   - query:      O(sqrt{n})
///   - update:     O(1)
/// Space: O(n)

struct SqrtDecomposition {
    int blk_size;
    vector<int> v, blk;

    SqrtDecomposition(vector<int> v) :
        v(v), blk(v.size())
    { init(); }

    void init() {
        build();
    }

    void update(int idx, int val) {
        blk[idx / blk_size] += val - v[idx];
        v[idx] = val;
    }

    int query(int l, int r) {
        int ans = 0;
        int cl = l/blk_size, cr = r/blk_size;

        if (cl == cr) {
            for (int i = l; i <= r; ++i)
                ans += v[i];
        } else {
            for (int i = l, end=(cl+1)*blk_size-1; i <= end; ++i)
                ans += v[i];
            for (int i = cl+1; i <= cr - 1; ++i)
                ans += blk[i];
            for (int i = cr*blk_size; i <= r; ++i)
                ans += v[i];
        }

        return ans;
    }

    void build() {
        int n = v.size();
        blk_size = (int) sqrt(n + 0.0) + 1;
        for (int i = 0; i < n; ++i)
            blk[idx / blk_size] += v[i];
    }
};
