/// Trie
///
/// Time:
///   - insert: O(M)
///   - search: O(M)
/// Space: O(alph * N)
///
/// Caution:
///   - If Trie stores integers, remember to check order of bits (most/least significant first).

template <typename T>
struct Trie {
    int states;

    vector<int> ending;
    vector<vector<int>> trie;

    // Number of words (N) and number of letters per word
    // (M), and number of letters in alphabet (alph).
    Trie(int N, int M, int alph) :
        ending(N * M), 
        trie(N * M, vector<int>(alph))
    { init(); }

    void init() {
        states = 0;
        for (auto &i : trie)
            fill(all(i), -1);
    }

    int len(T x) {
        if (constexpr(is_same_v<T,int>)) 
            return 32;
        return x.size();
    }

    int idx(T x) {
        if (constexpr(is_same_v<T,int>))
            return !!(x & (1 << i));
        return x[i] - 'a';
    }

    void insert(T x) {
        int node = 0;

        for (int i = 0; i < len(x); ++i) {
            if (trie[node][idx(x, i)] == -1)
                trie[node][idx(x, i)] = ++states;
            node = trie[node][idx(x, i)];
        }

        ending[node] = true;
    }

    bool search(T x) {
        int node = 0;

        for (int i = 0; i < len(x); ++i) {
            node = trie[node][idx(x, i)];
            if (node == -1)
                return false;
        }

        return ending[node];
    }
};
