/// Xenia and Tree

#include <bits/stdc++.h>

#define MAX 101010
#define EPS 1e-6
#define MOD 1000000007
#define inf 0x3f3f3f3f
#define llinf 0x3f3f3f3f3f3f3f3f

#define fi first
#define se second
#define pb push_back
#define ende '\n'

#define all(x) (x).begin(), (x).end()
#define rall(x) (x).rbegin(), (x).rend()
#define mset(x, y) memset(&x, (y), sizeof(x))

using namespace std; 

using ll = long long;
using ii = pair<int,int>;

#define MAXLOG 20

vector<ii> graph[MAX];

struct LCA {
    vector<int> h;
    vector<vector<int>> par, cost;

    LCA(int N) :
        h(N), 
        par(N, vector<int>(MAXLOG)), 
        cost(N, vector<int>(MAXLOG))
    {
        init();
    }

    void init() {
        for (auto &i : par)
            fill(all(i), -1);
        for (auto &i : cost)
            fill(all(i), 0);
        dfs(0);
    }

    inline int op(int a, int b) {
        return a + b;
    }

    void dfs(int v, int p = -1, int c = 0) {
        par[v][0] = p;
        cost[v][0] = c;

        if (p != -1)
            h[v] = h[p] + 1;

        for (int i = 1; i < MAXLOG; ++i)
            if (par[v][i - 1] != -1) {
                par[v][i] = par[par[v][i - 1]][i - 1];
                cost[v][i] = op(cost[v][i], op(cost[par[v][i-1]][i-1], cost[v][i-1]));
            }

        for (auto u : graph[v]) 
            if (p != u.fi)
                dfs(u.fi, v, u.se);
    }

    int query(int p, int q) {
        int ans = 0; // *

        if (h[p] < h[q])
            swap(p, q);

        for (int i = MAXLOG - 1; i >= 0; --i)
            if (par[p][i] != -1 && h[par[p][i]] >= h[q]) {
                ans = op(ans, cost[p][i]);
                p = par[p][i];
            }

        if (p == q) {
            return ans;
        }

        for (int i = MAXLOG - 1; i >= 0; --i)
            if (par[p][i] != -1 && par[p][i] != par[q][i]) {
                ans = op(ans, op(cost[p][i], cost[q][i]));
                p = par[p][i];
                q = par[q][i];
            }

        if (p == q) return ans;
        else return op(ans, op(cost[p][0], cost[q][0]));
    }
};

struct CentroidDecomposition {
    vector<int> par, size, marked;

    CentroidDecomposition(int N) :
        par(N), size(N), marked(N)
    {
        init();
    }

    void init() {
        fill(all(marked), 0);
        build(0, -1);
    }

    void build(int x, int p) {
        int n = dfs(x, -1);
        int centroid = get_centroid(x, -1, n);
        marked[centroid] = 1;

        par[centroid] = p;

        for (auto i : graph[centroid]) {
            if (!marked[i.fi])
                build(i.fi, centroid);
        }
    }

    int dfs(int x, int p) {
        size[x] = 1;
        for (auto i : graph[x])
            if (i.fi != p && !marked[i.fi])
                size[x] += dfs(i.fi, x);

        return size[x];
    }

    int get_centroid(int x, int p, int n) {
        for (auto i : graph[x])
            if (i.fi != p && size[i.fi] > n / 2 && !marked[i.fi])
                return get_centroid(i.fi, x, n);
        return x;
    }

    int operator[](int i) {
        return par[i];
    }
};

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);

    int n, m; cin >> n >> m;
    vector<int> ans(n, inf);

    for (int i = 0; i < n - 1; ++i) {
        int a, b; cin >> a >> b;
        a--, b--;
        graph[a].pb(ii(b, 1));
        graph[b].pb(ii(a, 1));
    }

    LCA lca(n);
    CentroidDecomposition cd(n);

    int p = 0;
    while (p != -1) {
        ans[p] = min(ans[p], lca.query(0, p));
        p = cd[p];
    }

    for (int i = 0; i < m; ++i) {
        int a, b; cin >> a >> b;
        b--;
        if (a == 1) {
            int p = b;
            while (p != -1) {
                ans[p] = min(ans[p], lca.query(b, p));
                p = cd[p];
            }
        } else {
            int p = b, res = inf;
            while (p != -1) {
                res = min(res, lca.query(b, p) + ans[p]);
                p = cd[p];
            }
            cout << res << ende;
        }
    }

    return 0;
}
