/// Trash Removal

#include <bits/stdc++.h>

#define EPS 1e-6
#define MOD 1000000007
#define inf 0x3f3f3f3f
#define llinf 0x3f3f3f3f3f3f3f3f

#define fi first
#define se second
#define pb push_back
#define ende '\n'

#define all(x) (x).begin(), (x).end()
#define rall(x) (x).rbegin(), (x).rend()
#define mset(x, y) memset(&x, (y), sizeof(x))

using namespace std; 

using ll = long long;
using ii = pair<int,int>;

template <typename T = double>
struct Point {
    T x, y; 

    Point() {}
    Point(T x, T y) : x(x), y(y) {}

    Point operator+(Point p) { return Point(x+p.x, y+p.y); }
    Point operator-(Point p) { return Point(x-p.x, y-p.y); }
    Point operator*(T s) { return Point(x*s, y*s); }

    T dot(Point p)   { return (x*p.x) + (y*p.y); }
    T cross(Point p) { return (x*p.y) - (y*p.x); }
};

bool cw(Point<> a, Point<> b, Point<> c) {
    return (b - a).cross(c - a) <= 0;
}

template <typename T = double>
struct Segment {
    Point<T> a, b;

    Segment(Point<T> a, Point<T> b) : a(a), b(b) {}

    double dist(Point<T> p) {
        return (a - b).cross(p - b)/sqrt((b - a).dot(b - a));
    }
};

vector<Point<>> convex_hull(vector<Point<>> &v) {
    int k = 0;
    vector<Point<>> ans(v.size() * 2);

    sort(all(v), [](const Point<> &a, const Point<> &b) {
            return (a.x == b.x) ? (a.y < b.y) : (a.x < b.x);
            });

    for (int i = 0; i < v.size(); ++i) {
        for (; k >= 2 && cw(ans[k-2], ans[k-1], v[i]); --k);
        ans[k++] = v[i];
    }

    for (int i = v.size() - 2, t = k + 1; i >= 0; --i) {
        for (; k >= t && cw(ans[k-2], ans[k-1], v[i]); --k);
        ans[k++] = v[i];
    }

    ans.resize(k);
    return ans;
}

double width(vector<Point<>> &v) {
    vector<Point<>> h = convex_hull(v);

    int n = h.size() - 1;
    double ans = 1e14;

    h[0] = h[n];
    for (int i = 1, j = 1; i <= n; ++i) {
        while ((h[i] - h[i-1]).cross(h[j%n+1] - h[i-1]) > 
                (h[i] - h[i-1]).cross(h[j] - h[i-1]))
            j = j % n + 1;

        Segment<> seg(h[i], h[i-1]);
        ans = min(ans, seg.dist(h[j]));
    }

    return ceil(ans*100)/100;
}

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);
    cout << setprecision(2) << fixed;

    int n, cas = 1;
    while (cin >> n && n) {
        vector<Point<>> v(n);
        for (int i = 0; i < n; ++i)
            cin >> v[i].x >> v[i].y;

        cout << "Case " << cas << ": " << width(v) << ende;
        cas++;
    }

    return 0;
}
