/// I Love Strings! (Aho-Corasick)

#include <bits/stdc++.h>

#define MAX 1010101
#define EPS 1e-6
#define MOD 1000000007
#define inf 0x3f3f3f3f
#define llinf 0x3f3f3f3f3f3f3f3f

#define fi first
#define se second
#define pb push_back
#define ende '\n'

#define all(x) (x).begin(), (x).end()
#define rall(x) (x).rbegin(), (x).rend()
#define mset(x, y) memset(&x, (y), sizeof(x))

using namespace std; 

using ll = long long;
using ii = pair<int,int>;

struct AhoCorasick {
    struct Node {
        int fail, occ, cnt;
        vector<int> words;
        map<char,int> next;

        Node() : fail(0), occ(-1), cnt(0) {}
        int has(char i) { return next.count(i); }
        int &operator[](char i) { return next[i]; }
    };

    vector<int> top;
    vector<Node> trie;

    AhoCorasick(const vector<string> &v) {
        trie.pb(Node());
        build(v);
        top = preprocess();
    }

    int insert(const string &s) {
        int n = 0;
        for (int i = 0; i < s.size(); n = trie[n][s[i]], ++i)
            if (!trie[n].has(s[i])) {
                trie[n][s[i]] = trie.size();
                trie.pb(Node());
            }
        return n;
    }

    void build(const vector<string> &v) {
        for (int i = 0; i < v.size(); ++i) {
            int n = insert(v[i]);
            trie[n].words.pb(i);
        }
    }

    inline int suffix(int v, char c) {
        while (v != 0 && !trie[v].has(c)) v = trie[v].fail;
        if (trie[v].has(c)) v = trie[v][c];
        return v;
    }

    vector<int> preprocess() {
        vector<int> Q = { 0 };
        for (int i = 0; i != Q.size(); ++i) {
            int u = Q[i];
            for (auto j : trie[u].next) {
                int &v = trie[j.se].fail;
                if (u) {
                    v = suffix(trie[u].fail, j.fi);
                    trie[j.se].occ = trie[v].words.size() ? v : trie[v].occ;
                } else {
                    v = trie[u].fail;
                    trie[j.se].occ = -1;
                }
                Q.pb(j.se);
            }
        }
        return Q;
    }

    vector<int> match(const string &p) {
        int u = 0;
        vector<int> ans;
        for (auto &i : p) {
            u = suffix(u, i);
            trie[u].cnt++;
        }

        for (int i = top.size() - 1; i >= 0; --i)
            trie[trie[top[i]].fail].cnt += trie[top[i]].cnt;

        for (auto i : trie)
            if (i.cnt && i.words.size())
                for (auto j : i.words) ans.pb(j);

        sort(all(ans));
        ans.erase(unique(all(ans)), ans.end());
        return ans;
    }
};

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);

    int t; cin >> t;
    while (t--) {
        string s; cin >> s;
        int n; cin >> n;
        vector<string> v(n);
        for (auto &i : v) cin >> i;

        AhoCorasick aho(v);
        vector<int> ans = aho.match(s);

        for (int i = 0, j = 0; i < n; ++i) {
            if (j < ans.size() && ans[j] == i) { 
                cout << 'y' << ende; ++j;
            } else
                cout << 'n' << ende;
        }
    }

    return 0;
}
