/// Query on a tree

#include <bits/stdc++.h>

#define MAX 10101
#define EPS 1e-6
#define MOD 1000000007
#define inf 0x3f3f3f3f
#define llinf 0x3f3f3f3f3f3f3f3f

#define fi first
#define se second
#define pb push_back
#define ende '\n'

#define all(x) (x).begin(), (x).end()
#define rall(x) (x).rbegin(), (x).rend()
#define mset(x, y) memset(&x, (y), sizeof(x))

using namespace std; 

using ll = long long;
using ii = pair<int,int>;

#define left(x) (x << 1)
#define right(x) ((x << 1) + 1)

int N;

template <typename T>
struct SegmentTree {
    using func = function<T(T,T)>;

    func op;
    T ident = T();
    vector<T> tree;

    SegmentTree(func op) : 
        op(op), tree(MAX*4) {}

    void build(const vector<T> &v, 
            int node = 1, int l = 0, int r = N - 1) 
    {
        if (l > r) 
            return;

        if (l == r)
            tree[node] = v[l];
        else {
            int m = (l + r) / 2;
            build(v, left(node), l, m);
            build(v, right(node), m + 1, r);
            tree[node] = op(tree[left(node)], tree[right(node)]);
        }
    }

    void update(int i, T val, 
            int node = 1, int l = 0, int r = N - 1) 
    {
        if (l > r || l > i || r < i) 
            return;

        if (l == r)
            tree[node] = val;
        else {
            int m = (l + r) / 2;
            update(i, val, left(node), l, m);
            update(i, val, right(node), m + 1, r);
            tree[node] = op(tree[left(node)], tree[right(node)]);
        }
    }

    T query(int i, int j, 
            int node = 1, int l = 0, int r = N - 1)
    {
        if (l > r || l > j || r < i) 
            return ident;

        if (l >= i && r <= j)
            return tree[node];

        int m = (l + r) / 2;
        T q1 = query(i, j, left(node), l, m);
        T q2 = query(i, j, right(node), m + 1, r);
        return op(q1, q2);
    }
};

ii edge[MAX];
vector<ii> graph[MAX];

template <typename ST>
struct HLD {
    ST &seg;
    int cnum, ptr;

    vector<int> dep, par, val;
    vector<int> head, heavy, pos, bot;

    HLD(int n, ST &seg) :
        seg(seg), 
        dep(n, 0), par(n), val(n),
        head(n), heavy(n, -1), pos(n), bot(n)
    {
        cnum = ptr = 0;

        N = n;
        dfs(0);
        decompose(0);

        for (int i = 0; i < n - 1; ++i)
            if (dep[edge[i].fi] > dep[edge[i].se])
                bot[i] = edge[i].fi;
            else
                bot[i] = edge[i].se;
    }

    int dfs(int x, int p = -1) {
        int size = 1;
        par[x] = p;

        int max_size = 0;
        for (auto i : graph[x])
            if (i.fi != p) {
                dep[i.fi] = dep[x] + 1;
                val[i.fi] = i.se;
                int isize = dfs(i.fi, x);

                size += isize;
                if (isize > max_size)
                    max_size = isize, heavy[x] = i.fi;
            }

        return size;
    }

    void decompose(int x, int h = 0) {
        head[x] = h;
        seg.update(ptr, val[x]);
        pos[x] = ptr++;

        if (heavy[x] != -1)
            decompose(heavy[x], h);

        for (auto i : graph[x])
            if (i.fi != par[x] && i.fi != heavy[x])
                decompose(i.fi, i.fi);
    }

    int query(int a, int b) {
        int ans = seg.ident;
        for (; head[a] != head[b]; b = par[head[b]]) {
            if (dep[head[a]] > dep[head[b]])
                swap(a, b);
            ans = seg.op(ans, seg.query(pos[head[b]], pos[b]));
        }

        if (dep[a] > dep[b])
            swap(a, b);

        return seg.op(ans, seg.query(pos[a] + 1, pos[b]));
    }

    void update(int i, int val) {
        seg.update(pos[bot[i]], val);
    }
};

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);

    int t; cin >> t;
    while (t--) {
        int n; cin >> n;
        for (int i = 0; i < n; ++i)
            graph[i].clear();

        for (int i = 0; i < n - 1; ++i) {
            int a, b, c; cin >> a >> b >> c;
            a--, b--;
            graph[a].pb(ii(b, c));
            graph[b].pb(ii(a, c));
            edge[i] = ii(a, b);
        }

        SegmentTree<int> seg([](int a, int b) { return max(a, b); });
        HLD<SegmentTree<int>> hld(n, seg);

        string op;
        while (cin >> op && op != "DONE") {
            if (op == "QUERY") {
                int a, b; cin >> a >> b; a--, b--;
                cout << hld.query(a, b) << ende;
            } else {
                int idx, w; cin >> idx >> w; idx--;
                hld.update(idx, w);
            }
        }
    }

    return 0;
}
