#include <bits/stdc++.h>

#define EPS 1e-6
#define MOD 1000000007
#define inf 0x3f3f3f3f
#define llinf 0x3f3f3f3f3f3f3f3f

#define fi first
#define se second
#define pb push_back
#define ende '\n'

#define all(x) (x).begin(), (x).end()
#define rall(x) (x).rbegin(), (x).rend()
#define mset(x, y) memset(&x, (y), sizeof(x))

using namespace std; 

using ll = long long;
using ii = pair<int,int>;

struct MinCostMaxFlow {
    struct Edge { 
        int u, v, cap; 
        double cost;
    };

    vector<Edge> edges;
    vector<vector<int>> adj;
    vector<int> vis, par, ind;
    vector<double> dist;

    MinCostMaxFlow(int N) :
        vis(N), dist(N), par(N), ind(N), adj(N) {}

    void add_edge(int u, int v, int cap, double cost) {
        adj[u].pb(edges.size());
        edges.pb({ u, v, cap, cost });

        adj[v].pb(edges.size());
        edges.pb({ v, u, 0, -cost });
    }

    bool spfa(int s, int t) {
        fill(all(dist), inf);
        dist[s] = 0;

        queue<int> Q;
        Q.push(s);

        while (!Q.empty()) {
            int u = Q.front(); Q.pop();
            vis[u] = 0;

            for (auto i : adj[u]) {
                Edge &e = edges[i];
                int v = e.v;

                if (e.cap > 0 && dist[v] > dist[u] + e.cost) {
                    dist[v] = dist[u] + e.cost;
                    par[v] = u;
                    ind[v] = i;

                    if (!vis[v]) {
                        Q.push(v);
                        vis[v] = 1;
                    }
                }
            }
        }

        return dist[t] < inf;
    }

    void run(int s, int t) {
        double min_cost = 0;
        int max_flow = 0;

        while (spfa(s, t)) {
            int flow = inf;
            for (int i = t; i != s; i = par[i])
                flow = min(flow, edges[ind[i]].cap);

            for (int i = t; i != s; i = par[i]) {
                edges[ind[i]  ].cap -= flow;
                edges[ind[i]^1].cap += flow;
            }

            min_cost += flow * dist[t];
            max_flow += flow;
        }
    }
};

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);

    int n; cin >> n;

    MinCostMaxFlow mcmf(2*n + 2);
    for (int i = 0; i < n; ++i) {
        mcmf.add_edge(0, i+1, 1, 0);
        for (int j = 0; j < n; ++j) {
            double x; cin >> x; 
            mcmf.add_edge(i+1, j+1+n, 1, -log(x));
        }
        mcmf.add_edge(i+n+1, 2*n+1, 1, 0);
    }

    vector<int> ans(n);
    mcmf.run(0, 2*n+1);
    for (int i = 0; i < mcmf.edges.size(); ++i)
        if (mcmf.edges[i].u > 0    && mcmf.edges[i].u <= n && 
                mcmf.edges[i].v >= n+1 && mcmf.edges[i].v < 2*n+1 && 
                mcmf.edges[i].cap == 0)
            ans[mcmf.edges[i].v-(n+1)] = mcmf.edges[i].u;

    for (auto i : ans)
        cout << i << " ";
    cout << ende;
    return 0;
}
